package sample;

/*************************************************************************
 * LICENSE AGREEMENT FOR SAMPLE CODE OR ADDITIONAL FUNCTIONALITY
 * SAS INSTITUTE INC. IS PROVIDING YOU WITH THE COMPUTER SOFTWARE CODE INCLUDED WITH
 * THIS AGREEMENT ("CODE") ON AN "AS IS" BASIS, AND AUTHORIZES YOU TO USE THE CODE SUBJECT
 * TO THE TERMS HEREOF. BY USING THE CODE, YOU AGREE TO THESE TERMS. YOUR USE OF THE CODE
 * IS AT YOUR OWN RISK. SAS INSTITUTE INC. MAKES NO REPRESENTATION OR WARRANTY, EXPRESS OR
 * IMPLIED, INCLUDING, BUT NOT LIMITED TO, WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, NONINFRINGEMENT AND TITLE, WITH RESPECT TO THE CODE.
 *
 * The Code is intended to be used solely as an example for use with part of a product ("Software")
 * you currently have licensed from SAS Institute Inc. or one of its subsidiaries or authorized
 * agents ("SAS"). The Code is designed to add functionality to the Software, but has not necessarily
 * been tested. Accordingly, SAS makes no representation or warranty that the Code will operate
 * error-free. SAS is under no obligation to maintain or support the Code.
 *
 * Neither SAS nor its licensors shall be liable to you or any third party for any general, special,
 * direct, indirect, consequential, incidental or other damages whatsoever arising out of or related
 * to your use or inability to use the Code, even if SAS has been advised of the possibility of such
 * damages.
 *
 * Except as otherwise provided above, the Code is governed by the same agreement that governs the
 * Software. If you do not have an existing agreement with SAS governing the Software, you may not
 * use the Code.
 *
 * SAS and all other SAS Institute Inc. product or service names are registered trademarks or
 * trademarks of SAS Institute Inc. in the USA and other countries. (r) indicates USA registration.
 * Other brand and product names are registered trademarks or trademarks of their respective companies.
 *
 * Copyright 2012, SAS Institute Inc., Cary, NC
 *************************************************************************/
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;

import com.sas.hls.client.Session;
import com.sas.hls.client.SessionFactory;
import com.sas.hls.client.exception.ConfigurationException;
import com.sas.hls.client.exception.HttpException;
import com.sas.hls.drug.client.DrugServiceManager;
import com.sas.hls.resource.repository.RepositoryFile;
import com.sas.hls.resource.repository.RepositoryResource;
import com.sas.hls.resource.repository.exception.RepositoryResourceNotFoundException;
import com.sas.hls.security.authentication.exception.AuthenticationException;



/**
 *
 * An example of how to use the API to check if a SAS LSAF resource is currently checked out.
 *
 */
public class SASLsafCheckoutState {

	/**
	 *
	 * @param argv
	 */
	public static void main(String[] argv) {
		URL serverURL = null;
		String serverURLString = null;

		String userid = null;
		char[] password = null;
		String resourcePath = null;

		if (argv.length < 1) {
			showUsage();
			System.exit(1);
		}

		for (int i = 0; i < argv.length; i++) {

			if (argv[i].equalsIgnoreCase("-s")) { //what server to talk to
				i++;
				if (i < argv.length) {
					serverURLString = argv[i];
				}
			} else if (argv[i].equalsIgnoreCase("-u")) { //what is my userid
				i++;
				if (i < argv.length) userid = argv[i];
			} else if (argv[i].equalsIgnoreCase("-p")) { //what is my password
				i++;
				if (i < argv.length) password = argv[i].toCharArray();

			} else { //what is the resource we want to check
				if (i < argv.length) {
					resourcePath = argv[i];
				} else {
					System.err.println("Error:  You did not specify a resource.");
					showUsage();
					System.exit(1);
				}
			}
		}

		//if serverURL not passed in arg list, prompt for it.
		if (serverURLString == null) {
			serverURLString = String.valueOf(readInput("Server URL: "));
		}

		try {   //validate URL given
			serverURL = new URL(serverURLString);
		} catch (MalformedURLException me) {
			System.err.println("Error:  The server URL is not valid.");
			showUsage();
			System.exit(1);
		}

		//get credential information if we didn't already get it from the program arguments
		if (userid == null || userid.length() == 0) {
			userid = String.valueOf(readInput("Username: "));
		}
		if (password == null || String.valueOf(password).length() == 0) {
			password = readInput("Password: ");
		}

		Session session = null;
		boolean connectionSuccessful = true;
		try{
			session = SessionFactory.newSession(serverURL, userid, String.valueOf(password));

			if(connectionSuccessful){
				RepositoryResource theResource = ((DrugServiceManager)session.getServiceManager()).getRepositoryResourceService().getResource(resourcePath);

				if(theResource instanceof RepositoryFile){
					RepositoryFile theFile = (RepositoryFile)theResource;
					if(theFile.isCheckedOut()){
						System.out.println("true");
					}else{
						System.out.println("false");
					}
				} else {
					System.err.println("Error:  Path specified is not a repository file.");
				}
				System.exit(0);
			}

		} catch (AuthenticationException ex) { // wrong uid/password, or account is locked etc...
			System.err.println(ex.getClass().getName() + "\n" + ex.getMessage());
			connectionSuccessful = false;
		} catch (HttpException ex) { // failure in communicating with server etc...
			System.err.println(ex.getClass().getName() + "\n" + ex.getMessage());
			connectionSuccessful = false;
		} catch (RepositoryResourceNotFoundException e) {
			System.err.println("Error:  Specified resource "+resourcePath+" is not found.");
		} catch (ConfigurationException e){
			//probably a mismatch between client and server versions of the API
			System.err.println("Configuration error: "+e.getLocalizedMessage());
		} finally {
			if(session != null){
				session.invalidate(); //release our connection
			}
		}

	}





	private static void showUsage(){
		System.out.println("Usage: SASLsafCheckoutState [-s <server URL>] [-u <username>] [-p <password>] <resource>");
		System.out.println("                ");
		System.out.println("       Returns:");
		System.out.println("             true or false if resource is checked out   ");



	}




	private static char[] readInput(String prompt) {
		char[] ret = new char[] {'\0'};
		try {
			BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
			System.out.print(prompt);
			ret = (in.readLine()).toCharArray();
		} catch (IOException e) {
			System.err.println("Error:  Error reading input.");
			System.exit(1);
		}
		return ret;
	}
}
